/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon

import org.commonmark.node.LinkReferenceDefinition
import org.commonmark.node.Node

/** @since 3.0.0 */
abstract class MarkwonReducer {
    abstract fun reduce(node: Node): List<Node>

    internal class DirectChildren : MarkwonReducer() {
        override fun reduce(root: Node): List<Node> {
            val list: List<Node>

            // we will extract all blocks that are direct children of Document
            var node = root.firstChild

            // please note, that if there are no children -> we will return a list with
            // single element (which was supplied)
            if (node == null) {
                list = listOf(root)
            } else {
                list = ArrayList()
                var temp: Node
                while (node != null) {
                    // @since 4.5.0 do not include LinkReferenceDefinition node (would result
                    //  in empty textView if rendered in recycler-view)
                    if (node !is LinkReferenceDefinition) {
                        list.add(node)
                    }
                    temp = node.next
                    node.unlink()
                    node = temp
                }
            }
            return list
        }
    }

    companion object {
        /**
         * @return direct children of supplied Node. In the most usual case will return all
         *   BlockNodes of a Document
         */
        fun directChildren(): MarkwonReducer {
            return DirectChildren()
        }
    }
}
