/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon

/**
 * Class to hold data in [RenderProps]. Represents a certain *property*.
 *
 * @param <T> represents the type that this instance holds
 * @see .of
 * @see .of
 * @since 3.0.0 </T>
 */
class Prop<T> internal constructor(private val name: String) {
    fun name(): String {
        return name
    }

    operator fun get(props: RenderProps): T? {
        return props[this]
    }

    operator fun get(props: RenderProps, defValue: T): T {
        return props[this, defValue]
    }

    fun require(props: RenderProps): T {
        return get(props) ?: throw NullPointerException(name)
    }

    operator fun set(props: RenderProps, value: T?) {
        props[this] = value
    }

    fun clear(props: RenderProps) {
        props.clear(this)
    }

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other == null || javaClass != other.javaClass) return false
        val prop = other as Prop<*>
        return name == prop.name
    }

    override fun hashCode(): Int {
        return name.hashCode()
    }

    override fun toString(): String {
        return "Prop{" + "name='" + name + '\'' + '}'
    }

    companion object {
        @Suppress("unused")
        fun <T> of(type: Class<T>, name: String): Prop<T> {
            return Prop(name)
        }

        @JvmStatic
        fun <T> of(name: String): Prop<T> {
            return Prop(name)
        }
    }
}
