/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.core

import io.noties.markwon.MarkwonVisitor
import org.commonmark.node.Node

/**
 * A [MarkwonVisitor.NodeVisitor] that ensures that a markdown block starts with a new line, all
 * children are visited and if further content available ensures a new line after self. Does not
 * render any spans
 *
 * @since 3.0.0
 */
class SimpleBlockNodeVisitor : MarkwonVisitor.NodeVisitor<Node> {
    override fun visit(visitor: MarkwonVisitor, node: Node) {
        visitor.blockStart(node)

        // @since 3.0.1 we keep track of start in order to apply spans (optionally)
        val length = visitor.length()
        visitor.visitChildren(node)

        // @since 3.0.1 we apply optional spans
        visitor.setSpansForNodeOptional(node, length)
        visitor.blockEnd(node)
    }
}
