/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.core.spans

import android.graphics.Canvas
import android.graphics.Paint
import android.text.Layout
import android.text.style.LeadingMarginSpan
import io.noties.markwon.core.MarkwonTheme

class BlockQuoteSpan(private val theme: MarkwonTheme) : LeadingMarginSpan {
    private val rect = ObjectsPool.rect()
    private val paint = ObjectsPool.paint()

    override fun getLeadingMargin(first: Boolean): Int {
        return theme.blockMargin
    }

    override fun drawLeadingMargin(
        c: Canvas,
        p: Paint,
        x: Int,
        dir: Int,
        top: Int,
        baseline: Int,
        bottom: Int,
        text: CharSequence,
        start: Int,
        end: Int,
        first: Boolean,
        layout: Layout
    ) {
        val width = theme.blockQuoteWidth
        paint.set(p)
        theme.applyBlockQuoteStyle(paint)
        val left: Int
        val right: Int
        run {
            val l = x + dir * width
            val r = l + dir * width
            left = l.coerceAtMost(r)
            right = l.coerceAtLeast(r)
        }
        rect[left, top, right] = bottom
        c.drawRect(rect, paint)
    }
}
