/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.core.spans

import android.text.Layout
import android.text.Spannable
import android.text.Spanned
import java.lang.ref.WeakReference

/** @since 4.4.0 */
class TextLayoutSpan internal constructor(layout: Layout) {
    private val reference: WeakReference<Layout>

    init {
        reference = WeakReference(layout)
    }

    fun layout(): Layout? {
        return reference.get()
    }

    companion object {
        /** @see .applyTo */
        fun layoutOf(cs: CharSequence): Layout? {
            return if (cs is Spanned) {
                layoutOf(cs)
            } else null
        }

        @JvmStatic
        fun layoutOf(spanned: Spanned): Layout? {
            val spans = spanned.getSpans(0, spanned.length, TextLayoutSpan::class.java)
            return if (spans != null && spans.isNotEmpty()) spans[0].layout() else null
        }

        @JvmStatic
        fun applyTo(spannable: Spannable, layout: Layout) {

            // remove all current ones (only one should be present)
            val spans = spannable.getSpans(0, spannable.length, TextLayoutSpan::class.java)
            if (spans != null) {
                for (span in spans) {
                    spannable.removeSpan(span)
                }
            }
            val span = TextLayoutSpan(layout)
            spannable.setSpan(span, 0, spannable.length, Spanned.SPAN_INCLUSIVE_INCLUSIVE)
        }
    }
}
