/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.core.spans

import android.text.Spannable
import android.text.Spanned
import android.widget.TextView
import java.lang.ref.WeakReference

/**
 * A special span that allows to obtain `TextView` in which spans are displayed
 *
 * @since 4.4.0
 */
class TextViewSpan(textView: TextView) {
    private val reference: WeakReference<TextView>

    init {
        reference = WeakReference(textView)
    }

    fun textView(): TextView? {
        return reference.get()
    }

    companion object {
        fun textViewOf(cs: CharSequence): TextView? {
            return if (cs is Spanned) {
                textViewOf(cs)
            } else null
        }

        @JvmStatic
        fun textViewOf(spanned: Spanned): TextView? {
            val spans = spanned.getSpans(0, spanned.length, TextViewSpan::class.java)
            return if (spans != null && spans.isNotEmpty()) spans[0].textView() else null
        }

        @JvmStatic
        fun applyTo(spannable: Spannable, textView: TextView) {
            val spans = spannable.getSpans(0, spannable.length, TextViewSpan::class.java)
            if (spans != null) {
                for (span in spans) {
                    spannable.removeSpan(span)
                }
            }
            val span = TextViewSpan(textView)
            // `SPAN_INCLUSIVE_INCLUSIVE` to persist in case of possible text change (deletion, etc)
            spannable.setSpan(span, 0, spannable.length, Spanned.SPAN_INCLUSIVE_INCLUSIVE)
        }
    }
}
