/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.ext.strikethrough;

import android.text.style.StrikethroughSpan;
import androidx.annotation.NonNull;
import io.noties.markwon.AbstractMarkwonPlugin;
import io.noties.markwon.MarkwonSpansFactory;
import io.noties.markwon.MarkwonVisitor;
import java.util.Collections;
import org.commonmark.ext.gfm.strikethrough.Strikethrough;
import org.commonmark.ext.gfm.strikethrough.StrikethroughExtension;
import org.commonmark.parser.Parser;

/**
 * Plugin to add strikethrough markdown feature. This plugin will extend commonmark-java.Parser with
 * strikethrough extension, add SpanFactory and register commonmark-java.Strikethrough node visitor
 *
 * @see #create()
 * @since 3.0.0
 */
public class StrikethroughPlugin extends AbstractMarkwonPlugin {

  @NonNull
  public static StrikethroughPlugin create() {
    return new StrikethroughPlugin();
  }

  @Override
  public void configureParser(@NonNull Parser.Builder builder) {
    builder.extensions(Collections.singleton(StrikethroughExtension.create()));
  }

  @Override
  public void configureSpansFactory(@NonNull MarkwonSpansFactory.Builder builder) {
    builder.setFactory(Strikethrough.class, (configuration, props) -> new StrikethroughSpan());
  }

  @Override
  public void configureVisitor(@NonNull MarkwonVisitor.Builder builder) {
    builder.on(
        Strikethrough.class,
        (visitor, strikethrough) -> {
          final int length = visitor.length();
          visitor.visitChildren(strikethrough);
          visitor.setSpansForNodeOptional(strikethrough, length);
        });
  }
}
