/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.ext.tables;

import android.text.Spanned;
import android.text.TextUtils;
import android.view.View;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import io.noties.markwon.R;

abstract class TableRowsScheduler {

  private TableRowsScheduler() {}

  static void schedule(@NonNull final TextView view) {
    final Object[] spans = extract(view);
    if (spans != null && spans.length > 0) {

      if (view.getTag(R.id.markwon_tables_scheduler) == null) {
        final View.OnAttachStateChangeListener listener =
            new View.OnAttachStateChangeListener() {
              @Override
              public void onViewAttachedToWindow(View v) {}

              @Override
              public void onViewDetachedFromWindow(View v) {
                unschedule(view);
                view.removeOnAttachStateChangeListener(this);
                view.setTag(R.id.markwon_tables_scheduler, null);
              }
            };
        view.addOnAttachStateChangeListener(listener);
        view.setTag(R.id.markwon_tables_scheduler, listener);
      }

      final TableRowSpan.Invalidator invalidator =
          new TableRowSpan.Invalidator() {

            // @since 4.1.0
            // let's stack-up invalidation calls (so invalidation happens,
            // but not with each table-row-span draw call)
            final Runnable runnable = () -> view.setText(view.getText());

            @Override
            public void invalidate() {
              // @since 4.1.0 post invalidation (combine multiple calls)
              view.removeCallbacks(runnable);
              view.post(runnable);
            }
          };

      for (Object span : spans) {
        ((TableRowSpan) span).invalidator(invalidator);
      }
    }
  }

  static void unschedule(@NonNull TextView view) {
    final Object[] spans = extract(view);
    if (spans != null) {
      for (Object span : spans) {
        ((TableRowSpan) span).invalidator(null);
      }
    }
  }

  @Nullable
  private static Object[] extract(@NonNull TextView view) {
    final Object[] out;
    final CharSequence text = view.getText();
    if (!TextUtils.isEmpty(text) && text instanceof Spanned) {
      out = ((Spanned) text).getSpans(0, text.length(), TableRowSpan.class);
    } else {
      out = null;
    }
    return out;
  }
}
