/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.html

/**
 * This class will be used to append some text to output in order to apply a Span for this tag.
 * Please note that this class will be used for _void_ tags and tags that are self-closed (even if
 * HTML spec doesn\'t specify a tag as self-closed). This is due to the fact that underlying parser
 * does not validate context and does not check if a tag is correctly used. Plus it will be used for
 * tags without content, for example: `<my-custom-element></my-custom-element>`
 *
 * @since 2.0.0
 */
class HtmlEmptyTagReplacement {
    /**
     * @return replacement for supplied startTag or null if no replacement should occur (which will
     *   lead to `Inline` tag have start &amp; end the same value, thus not applicable for applying
     *   a Span)
     */
    fun replace(tag: HtmlTag): String? {
        val replacement: String?
        val name = tag.name()
        replacement =
            if ("br" == name) {
                "\n"
            } else if ("img" == name) {
                val alt = tag.attributes()["alt"]
                if (alt == null || alt.isEmpty()) {
                    // no alt is provided
                    IMG_REPLACEMENT
                } else {
                    alt
                }
            } else if ("iframe" == name) {
                // @since 4.4.0 make iframe non-empty
                IFRAME_REPLACEMENT
            } else {
                null
            }
        return replacement
    }

    companion object {
        private const val IMG_REPLACEMENT = "\uFFFC"
        private const val IFRAME_REPLACEMENT = "\u00a0" // non-breakable space

        @JvmStatic
        fun create(): HtmlEmptyTagReplacement {
            return HtmlEmptyTagReplacement()
        }
    }
}
