/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.html.jsoup.parser

/**
 * A Parse Error records an error in the input HTML that occurs in either the tokenisation or the
 * tree building phase.
 */
class ParseError {
    /**
     * Retrieves the offset of the error.
     *
     * @return error offset within input
     */
    var position: Int
        private set

    /**
     * Retrieve the error message.
     *
     * @return the error message.
     */
    var errorMessage: String
        private set

    internal constructor(pos: Int, errorMsg: String) {
        position = pos
        errorMessage = errorMsg
    }

    internal constructor(pos: Int, errorFormat: String?, vararg args: Any?) {
        errorMessage = String.format(errorFormat!!, *args)
        position = pos
    }

    override fun toString(): String {
        return "$position: $errorMessage"
    }
}
