/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.html.tag

import android.text.TextUtils
import io.noties.markwon.MarkwonConfiguration
import io.noties.markwon.RenderProps
import io.noties.markwon.html.CssInlineStyleParser
import io.noties.markwon.html.HtmlTag
import io.noties.markwon.image.ImageProps
import io.noties.markwon.image.ImageSize
import org.commonmark.node.Image

class ImageHandler internal constructor(private val imageSizeParser: ImageSizeParser) :
    SimpleTagHandler() {
    override fun supportedTags(): Collection<String> {
        return setOf("img")
    }

    override fun getSpans(
        configuration: MarkwonConfiguration,
        renderProps: RenderProps,
        tag: HtmlTag
    ): Any? {
        val attributes = tag.attributes()
        val src = attributes["src"]
        if (TextUtils.isEmpty(src)) {
            return null
        }
        val spanFactory = configuration.spansFactory()[Image::class.java] ?: return null
        val destination = configuration.imageDestinationProcessor().process(src!!)
        val imageSize = imageSizeParser.parse(tag.attributes())

        // todo: replacement text is link... as we are not at block level
        // and cannot inspect the parent of this node... (img and a are both inlines)
        //
        // but we can look and see if we are inside a LinkSpan (will have to extend TagHandler
        // to obtain an instance SpannableBuilder for inspection)
        ImageProps.DESTINATION[renderProps] = destination
        ImageProps.IMAGE_SIZE[renderProps] = imageSize
        ImageProps.REPLACEMENT_TEXT_IS_LINK[renderProps] = false
        return spanFactory.getSpans(configuration, renderProps)
    }

    internal interface ImageSizeParser {
        fun parse(attributes: Map<String?, String?>): ImageSize?
    }

    companion object {
        @JvmStatic
        fun create(): ImageHandler {
            return ImageHandler(ImageSizeParserImpl(CssInlineStyleParser.create()))
        }
    }
}
