/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.html.tag

import io.noties.markwon.MarkwonVisitor
import io.noties.markwon.SpannableBuilder
import io.noties.markwon.core.CoreProps
import io.noties.markwon.html.HtmlTag
import io.noties.markwon.html.MarkwonHtmlRenderer
import io.noties.markwon.html.TagHandler
import org.commonmark.node.ListItem

class ListHandler : TagHandler() {
    override fun handle(visitor: MarkwonVisitor, renderer: MarkwonHtmlRenderer, tag: HtmlTag) {
        if (!tag.isBlock) {
            return
        }
        val block = tag.asBlock
        val ol = "ol" == block.name()
        val ul = "ul" == block.name()
        if (!ol && !ul) {
            return
        }
        val configuration = visitor.configuration()
        val renderProps = visitor.renderProps()
        val spanFactory = configuration.spansFactory()[ListItem::class.java]
        var number = 1
        val bulletLevel = currentBulletListLevel(block)
        for (child in block.children()) {
            visitChildren(visitor, renderer, child)
            if (spanFactory != null && "li" == child.name()) {

                // insert list item here
                if (ol) {
                    CoreProps.LIST_ITEM_TYPE[renderProps] = CoreProps.ListItemType.ORDERED
                    CoreProps.ORDERED_LIST_ITEM_NUMBER[renderProps] = number++
                } else {
                    CoreProps.LIST_ITEM_TYPE[renderProps] = CoreProps.ListItemType.BULLET
                    CoreProps.BULLET_LIST_ITEM_LEVEL[renderProps] = bulletLevel
                }
                SpannableBuilder.setSpans(
                    visitor.builder(),
                    spanFactory.getSpans(configuration, renderProps),
                    child.start(),
                    child.end())
            }
        }
    }

    override fun supportedTags(): Collection<String> {
        return mutableListOf("ol", "ul")
    }

    companion object {
        private fun currentBulletListLevel(block: HtmlTag.Block): Int {
            var block = block
            var level = 0
            while (block.parent().also { block = it!! } != null) {
                if ("ul" == block.name() || "ol" == block.name()) {
                    level += 1
                }
            }
            return level
        }
    }
}
