/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.html.tag

import android.text.style.StrikethroughSpan
import io.noties.markwon.MarkwonVisitor
import io.noties.markwon.SpannableBuilder
import io.noties.markwon.html.HtmlTag
import io.noties.markwon.html.MarkwonHtmlRenderer
import io.noties.markwon.html.TagHandler
import org.commonmark.ext.gfm.strikethrough.Strikethrough

class StrikeHandler : TagHandler() {
    override fun handle(visitor: MarkwonVisitor, renderer: MarkwonHtmlRenderer, tag: HtmlTag) {
        if (tag.isBlock) {
            visitChildren(visitor, renderer, tag.asBlock)
        }
        SpannableBuilder.setSpans(
            visitor.builder(),
            if (HAS_MARKDOWN_IMPLEMENTATION) getMarkdownSpans(visitor) else StrikethroughSpan(),
            tag.start(),
            tag.end())
    }

    override fun supportedTags(): Collection<String> {
        return mutableListOf("s", "del")
    }

    companion object {
        // flag to detect if commonmark-java-strikethrough is in classpath, so we use SpanFactory
        // to obtain strikethrough span
        private var HAS_MARKDOWN_IMPLEMENTATION = false

        init {
            val hasMarkdownImplementation: Boolean =
                try {
                    // @since 4.3.1 we class Class.forName instead of trying
                    //  to access the class by full qualified name (which caused issues with
                    // DexGuard)
                    Class.forName("org.commonmark.ext.gfm.strikethrough.Strikethrough")
                    true
                } catch (t: Throwable) {
                    false
                }
            HAS_MARKDOWN_IMPLEMENTATION = hasMarkdownImplementation
        }

        private fun getMarkdownSpans(visitor: MarkwonVisitor): Any? {
            val configuration = visitor.configuration()
            val spanFactory = configuration.spansFactory()[Strikethrough::class.java] ?: return null
            return spanFactory.getSpans(configuration, visitor.renderProps())
        }
    }
}
