/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.image

import android.content.res.Resources
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import java.io.InputStream

/**
 * This class can be used as the last [MediaDecoder] to _try_ to handle all rest cases. Here we just
 * assume that supplied InputStream is of image type and try to decode it.
 *
 * **NB** if you are dealing with big images that require down scaling see
 * [DefaultDownScalingMediaDecoder] which additionally down scales displayed images.
 *
 * @see DefaultDownScalingMediaDecoder
 * @since 1.1.0
 */
class DefaultMediaDecoder internal constructor(private val resources: Resources) : MediaDecoder() {
    override fun decode(contentType: String?, inputStream: InputStream): Drawable {
        val bitmap: Bitmap =
            try {
                // absolutely not optimal... thing
                BitmapFactory.decodeStream(inputStream)
            } catch (t: Throwable) {
                throw IllegalStateException("Exception decoding input-stream", t)
            }
        return BitmapDrawable(resources, bitmap)
    }

    override fun supportedTypes(): Collection<String> {
        return emptySet()
    }

    companion object {
        fun create(): DefaultMediaDecoder {
            return DefaultMediaDecoder(Resources.getSystem())
        }

        fun create(resources: Resources): DefaultMediaDecoder {
            return DefaultMediaDecoder(resources)
        }
    }
}
