/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.image

import android.graphics.Rect

/** @since 1.0.1 */
@Suppress("unused")
open class ImageSizeResolverDef : ImageSizeResolver() {
    override fun resolveImageSize(drawable: AsyncDrawable): Rect {
        return resolveImageSize(
            drawable.imageSize,
            drawable.result.bounds,
            drawable.lastKnownCanvasWidth,
            drawable.lastKnowTextSize)
    }

    private fun resolveImageSize(
        imageSize: ImageSize?,
        imageBounds: Rect,
        canvasWidth: Int,
        textSize: Float
    ): Rect {
        if (imageSize == null) {
            // @since 2.0.0 post process bounds to fit canvasWidth (previously was inside
            // AsyncDrawable)
            //      must be applied only if imageSize is null
            val rect: Rect
            val w = imageBounds.width()
            rect =
                if (w > canvasWidth) {
                    val reduceRatio = w.toFloat() / canvasWidth
                    Rect(0, 0, canvasWidth, (imageBounds.height() / reduceRatio + .5f).toInt())
                } else {
                    imageBounds
                }
            return rect
        }
        val rect: Rect
        val width = imageSize.width
        val height = imageSize.height
        val imageWidth = imageBounds.width()
        val imageHeight = imageBounds.height()
        val ratio = imageWidth.toFloat() / imageHeight
        if (width != null) {
            val h: Int
            val w: Int =
                if (UNIT_PERCENT == width.unit) {
                    (canvasWidth * (width.value / 100f) + .5f).toInt()
                } else {
                    resolveAbsolute(width, imageWidth, textSize)
                }
            h =
                if (height == null || UNIT_PERCENT == height.unit) {
                    (w / ratio + .5f).toInt()
                } else {
                    resolveAbsolute(height, imageHeight, textSize)
                }
            rect = Rect(0, 0, w, h)
        } else if (height != null) {
            rect =
                if (UNIT_PERCENT != height.unit) {
                    val h = resolveAbsolute(height, imageHeight, textSize)
                    val w = (h * ratio + .5f).toInt()
                    Rect(0, 0, w, h)
                } else {
                    imageBounds
                }
        } else {
            rect = imageBounds
        }
        return rect
    }

    private fun resolveAbsolute(
        dimension: ImageSize.Dimension,
        original: Int,
        textSize: Float
    ): Int {
        val out: Int =
            if (UNIT_EM == dimension.unit) {
                (dimension.value * textSize + .5f).toInt()
            } else {
                (dimension.value + .5f).toInt()
            }
        return out
    }

    companion object {
        // we track these two, others are considered to be pixels
        protected const val UNIT_PERCENT = "%"
        protected const val UNIT_EM = "em"
    }
}
