/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.image.data

import android.text.TextUtils
import android.util.Base64

abstract class DataUriDecoder {
    @Throws(Throwable::class) abstract fun decode(dataUri: DataUri): ByteArray?

    internal class Impl : DataUriDecoder() {
        @Throws(Throwable::class)
        override fun decode(dataUri: DataUri): ByteArray? {
            val data = dataUri.data()
            return if (!TextUtils.isEmpty(data)) {
                if (dataUri.base64()) {
                    Base64.decode(data!!.toByteArray(charset(CHARSET)), Base64.DEFAULT)
                } else {
                    data!!.toByteArray(charset(CHARSET))
                }
            } else {
                null
            }
        }

        companion object {
            private const val CHARSET = "UTF-8"
        }
    }

    companion object {
        fun create(): DataUriDecoder {
            return Impl()
        }
    }
}
