/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.image.data

import android.net.Uri
import io.noties.markwon.image.ImageItem
import io.noties.markwon.image.SchemeHandler
import java.io.ByteArrayInputStream

/** @since 2.0.0 */
class DataUriSchemeHandler
internal constructor(private val uriParser: DataUriParser, private val uriDecoder: DataUriDecoder) :
    SchemeHandler() {
    override fun handle(raw: String, uri: Uri): ImageItem {
        check(raw.startsWith(START)) { "Invalid data-uri: $raw" }
        val part = raw.substring(START.length)
        val dataUri = uriParser.parse(part) ?: throw IllegalStateException("Invalid data-uri: $raw")
        val bytes: ByteArray? =
            try {
                uriDecoder.decode(dataUri)
            } catch (t: Throwable) {
                throw IllegalStateException("Cannot decode data-uri: $raw", t)
            }
        checkNotNull(bytes) { "Decoding data-uri failed: $raw" }
        return ImageItem.withDecodingNeeded(dataUri.contentType(), ByteArrayInputStream(bytes))
    }

    override fun supportedSchemes(): Collection<String> {
        return setOf(SCHEME)
    }

    companion object {
        const val SCHEME = "data"
        private const val START = "data:"

        @JvmStatic
        fun create(): DataUriSchemeHandler {
            return DataUriSchemeHandler(DataUriParser.create(), DataUriDecoder.create())
        }
    }
}
