/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.image.destination

import android.net.Uri
import android.text.TextUtils

/**
 * [ImageDestinationProcessor] that treats all destinations **without scheme** information as
 * pointing to the `assets` folder of an application. Please note that this processor only adds
 * required `file:///android_asset/` prefix to destinations and actual image loading must take that
 * into account (implement this functionality).
 *
 * `FileSchemeHandler` from the `image` module supports asset images when created with
 * `createWithAssets` factory method
 *
 * @since 4.4.0
 */
class ImageDestinationProcessorAssets
@JvmOverloads
constructor(private val processor: ImageDestinationProcessor? = null) :
    ImageDestinationProcessor() {
    private val assetsProcessor = ImageDestinationProcessorRelativeToAbsolute(MOCK)

    override fun process(destination: String): String {
        val out: String
        val uri = Uri.parse(destination)
        out =
            if (TextUtils.isEmpty(uri.scheme)) {
                assetsProcessor.process(destination).replace(MOCK, BASE)
            } else {
                processor?.process(destination) ?: destination
            }
        return out
    }

    companion object {
        const val MOCK = "https://android.asset/"
        const val BASE = "file:///android_asset/"

        fun create(parent: ImageDestinationProcessor?): ImageDestinationProcessorAssets {
            return ImageDestinationProcessorAssets(parent)
        }
    }
}
