/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.image.gif

import android.graphics.drawable.Drawable
import io.noties.markwon.image.MediaDecoder
import java.io.ByteArrayOutputStream
import java.io.IOException
import java.io.InputStream
import pl.droidsonroids.gif.GifDrawable

/** @since 1.1.0 */
open class GifMediaDecoder protected constructor(private val autoPlayGif: Boolean) :
    MediaDecoder() {
    init {

        // @since 4.0.0
        validate()
    }

    override fun decode(contentType: String?, inputStream: InputStream): Drawable {
        val bytes: ByteArray =
            try {
                readBytes(inputStream)
            } catch (e: IOException) {
                throw IllegalStateException("Cannot read GIF input-stream", e)
            }
        val drawable: GifDrawable =
            try {
                newGifDrawable(bytes)
            } catch (e: IOException) {
                throw IllegalStateException("Exception creating GifDrawable", e)
            }
        if (!autoPlayGif) {
            drawable.pause()
        }
        return drawable
    }

    override fun supportedTypes(): Collection<String> {
        return setOf(CONTENT_TYPE)
    }

    @Throws(IOException::class)
    protected fun newGifDrawable(bytes: ByteArray): GifDrawable {
        return GifDrawable(bytes)
    }

    companion object {
        const val CONTENT_TYPE = "image/gif"

        /**
         * Creates a [GifMediaDecoder] with `autoPlayGif = true`
         *
         * @since 4.0.0
         */
        @JvmOverloads
        fun create(autoPlayGif: Boolean = true): GifMediaDecoder {
            return GifMediaDecoder(autoPlayGif)
        }

        @Throws(IOException::class)
        protected fun readBytes(stream: InputStream): ByteArray {
            val outputStream = ByteArrayOutputStream()
            val length = 1024 * 8
            val buffer = ByteArray(length)
            var read: Int
            while (stream.read(buffer, 0, length).also { read = it } != -1) {
                outputStream.write(buffer, 0, read)
            }
            return outputStream.toByteArray()
        }

        private fun validate() {
            check(GifSupport.hasGifSupport()) { GifSupport.missingMessage() }
        }
    }
}
