/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.image.network

import android.net.Uri
import io.noties.markwon.image.ImageItem
import io.noties.markwon.image.SchemeHandler
import java.io.BufferedInputStream
import java.io.IOException
import java.io.InputStream
import java.net.HttpURLConnection
import java.net.URL

/**
 * A simple network scheme handler that is not dependent on any external libraries.
 *
 * @see .create
 * @since 3.0.0
 */
class NetworkSchemeHandler internal constructor() : SchemeHandler() {
    override fun handle(raw: String, uri: Uri): ImageItem {
        val imageItem: ImageItem =
            try {
                val url = URL(raw)
                val connection = url.openConnection() as HttpURLConnection
                connection.connect()
                val responseCode = connection.responseCode
                if (responseCode in 200..299) {
                    val contentType = contentType(connection.getHeaderField("Content-Type"))
                    val inputStream: InputStream = BufferedInputStream(connection.inputStream)
                    ImageItem.withDecodingNeeded(contentType, inputStream)
                } else {
                    throw IOException("Bad response code: $responseCode, url: $raw")
                }
            } catch (e: IOException) {
                throw IllegalStateException("Exception obtaining network resource: $raw", e)
            }
        return imageItem
    }

    override fun supportedSchemes(): Collection<String> {
        return listOf(SCHEME_HTTP, SCHEME_HTTPS)
    }

    companion object {
        const val SCHEME_HTTP = "http"
        const val SCHEME_HTTPS = "https"

        @JvmStatic
        fun create(): NetworkSchemeHandler {
            return NetworkSchemeHandler()
        }

        fun contentType(contentType: String?): String? {
            if (contentType == null) {
                return null
            }
            val index = contentType.indexOf(';')
            return if (index > -1) {
                contentType.substring(0, index)
            } else contentType
        }
    }
}
