/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.image.network

import android.net.Uri
import io.noties.markwon.image.ImageItem
import io.noties.markwon.image.SchemeHandler
import okhttp3.Call
import okhttp3.OkHttpClient
import okhttp3.Request
import okhttp3.Request.Builder
import okhttp3.Response

/** @since 4.0.0 */
class OkHttpNetworkSchemeHandler internal constructor(factory: Call.Factory) : SchemeHandler() {
    // @since 4.0.0, previously just OkHttpClient
    private val factory: Call.Factory

    init {
        this.factory = factory
    }

    override fun handle(raw: String, uri: Uri): ImageItem {
        val request: Request = Builder().url(raw).tag(raw).build()
        val response: Response =
            try {
                factory.newCall(request).execute()
            } catch (t: Throwable) {
                throw IllegalStateException("Exception obtaining network resource: $raw", t)
            }
        val body = response.body
        val inputStream =
            body?.byteStream()
                ?: throw IllegalStateException("Response does not contain body: $raw")

        // important to process content-type as it can have encoding specified (which we should
        // remove)
        val contentType: String? =
            NetworkSchemeHandler.contentType(response.header(HEADER_CONTENT_TYPE))
        return ImageItem.withDecodingNeeded(contentType, inputStream)
    }

    override fun supportedSchemes(): Collection<String> {
        return listOf(NetworkSchemeHandler.SCHEME_HTTP, NetworkSchemeHandler.SCHEME_HTTPS)
    }

    companion object {
        private const val HEADER_CONTENT_TYPE = "Content-Type"

        /** @see .create */
        @JvmOverloads
        fun create(client: OkHttpClient = OkHttpClient()): OkHttpNetworkSchemeHandler {
            // explicit cast, otherwise a recursive call
            return create(client as Call.Factory)
        }

        /** @since 4.0.0 */
        fun create(factory: Call.Factory): OkHttpNetworkSchemeHandler {
            return OkHttpNetworkSchemeHandler(factory)
        }
    }
}
