/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.image.svg

import android.content.res.Resources
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import com.caverock.androidsvg.SVG
import com.caverock.androidsvg.SVGParseException
import io.noties.markwon.image.MediaDecoder
import java.io.InputStream

/** @since 1.1.0 */
class SvgMediaDecoder internal constructor(private val resources: Resources) : MediaDecoder() {
    init {

        // @since 4.0.0
        validate()
    }

    override fun decode(contentType: String?, inputStream: InputStream): Drawable {
        val svg: SVG =
            try {
                SVG.getFromInputStream(inputStream)
            } catch (e: SVGParseException) {
                throw IllegalStateException("Exception decoding SVG", e)
            }
        val w = svg.documentWidth
        val h = svg.documentHeight
        val density = resources.displayMetrics.density
        val width = (w * density + .5f).toInt()
        val height = (h * density + .5f).toInt()
        val bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_4444)
        val canvas = Canvas(bitmap)
        canvas.scale(density, density)
        svg.renderToCanvas(canvas)
        return BitmapDrawable(resources, bitmap)
    }

    override fun supportedTypes(): Collection<String> {
        return setOf(CONTENT_TYPE)
    }

    companion object {
        const val CONTENT_TYPE = "image/svg+xml"

        /**
         * @see .create
         * @since 4.0.0
         */
        @JvmOverloads
        fun create(resources: Resources = Resources.getSystem()): SvgMediaDecoder {
            return SvgMediaDecoder(resources)
        }

        private fun validate() {
            check(SvgSupport.hasSvgSupport()) { SvgSupport.missingMessage() }
        }
    }
}
