/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.image.svg

import android.graphics.drawable.Drawable
import android.graphics.drawable.PictureDrawable
import com.caverock.androidsvg.SVG
import com.caverock.androidsvg.SVGParseException
import io.noties.markwon.image.MediaDecoder
import java.io.InputStream

/** @since 4.2.0 */
class SvgPictureMediaDecoder : MediaDecoder() {
    override fun decode(contentType: String?, inputStream: InputStream): Drawable {
        val svg: SVG =
            try {
                SVG.getFromInputStream(inputStream)
            } catch (e: SVGParseException) {
                throw IllegalStateException("Exception decoding SVG", e)
            }
        val picture = svg.renderToPicture()
        return PictureDrawable(picture)
    }

    override fun supportedTypes(): Collection<String> {
        return setOf(CONTENT_TYPE)
    }

    companion object {
        const val CONTENT_TYPE = "image/svg+xml"

        fun create(): SvgPictureMediaDecoder {
            return SvgPictureMediaDecoder()
        }
    }
}
