/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package io.noties.markwon.utils

import android.text.Layout

/** @since 4.4.0 */
object LayoutUtils {
    private const val DEFAULT_EXTRA = 0f
    private const val DEFAULT_MULTIPLIER = 1f

    fun getLineBottomWithoutPaddingAndSpacing(layout: Layout, line: Int): Int {
        val bottom = layout.getLineBottom(line)
        val lastLineSpacingNotAdded = true
        val isSpanLastLine = line == layout.lineCount - 1
        val lineBottom: Int
        val lineSpacingExtra = layout.spacingAdd
        val lineSpacingMultiplier = layout.spacingMultiplier

        // simplified check
        val hasLineSpacing =
            (lineSpacingExtra != DEFAULT_EXTRA || lineSpacingMultiplier != DEFAULT_MULTIPLIER)
        lineBottom =
            if (!hasLineSpacing || isSpanLastLine && lastLineSpacingNotAdded) {
                bottom
            } else {
                val extra: Float =
                    if (DEFAULT_MULTIPLIER.compareTo(lineSpacingMultiplier) != 0) {
                        val lineHeight = getLineHeight(layout, line)
                        lineHeight - (lineHeight - lineSpacingExtra) / lineSpacingMultiplier
                    } else {
                        lineSpacingExtra
                    }
                (bottom - extra + .5f).toInt()
            }

        // check if it is the last line that span is occupying **and** that this line is the last
        //  one in TextView
        return if (isSpanLastLine && line == layout.lineCount - 1) {
            lineBottom - layout.bottomPadding
        } else lineBottom
    }

    fun getLineTopWithoutPadding(layout: Layout, line: Int): Int {
        val top = layout.getLineTop(line)
        return if (line == 0) {
            top - layout.topPadding
        } else top
    }

    fun getLineHeight(layout: Layout, line: Int): Int {
        return layout.getLineTop(line + 1) - layout.getLineTop(line)
    }
}
